Feature: Attachments

  Scenario: Attach a buffer
    Given a file named "features/a.feature" with:
      """
      Feature: some feature

      Scenario: I've declared one step and it is passing
          Given This step is passing
      """
    And a file named "features/step_definitions/cucumber_steps.js" with:
      """
      var cucumberSteps = function() {
        this.Given(/^This step is passing$/, function(callback) { callback(); });
      };
      module.exports = cucumberSteps;
      """
    And a file named "features/support/hooks.js" with:
      """
      var hooks = function () {
        this.Before(function(scenario, callback) {
          scenario.attach(new Buffer([137, 80, 78, 71]), 'image/png');
          callback();
        });
      };

      module.exports = hooks;
      """
    When I run cucumber.js with `-f json`
    Then it outputs this json:
      """
      [
        {
          "id": "some-feature",
          "name": "some feature",
          "tags": [],
          "line": 1,
          "keyword": "Feature",
          "uri": "<current-directory>/features/a.feature",
          "elements": [
            {
              "name": "I've declared one step and it is passing",
              "id": "some-feature;i've-declared-one-step-and-it-is-passing",
              "line": 3,
              "keyword": "Scenario",
              "tags": [],
              "type": "scenario",
              "steps": [
                {
                  "keyword": "Before ",
                  "hidden": true,
                  "result": {
                    "duration": "<duration>",
                    "status": "passed"
                  },
                  "arguments": [],
                  "match": {
                    "location": "<current-directory>/features/support/hooks.js:2"
                  },
                  "embeddings": [
                    {
                      "mime_type": "image/png",
                      "data": "iVBORw=="
                    }
                  ]
                },
                {
                  "name": "This step is passing",
                  "line": 4,
                  "keyword": "Given ",
                  "result": {
                    "duration": "<duration>",
                    "status": "passed"
                  },
                  "arguments": [],
                  "match": {
                    "location": "<current-directory>/features/step_definitions/cucumber_steps.js:2"
                  }
                }
              ]
            }
          ]
        }
      ]
      """

  Scenario: Attach a stream
    Given a file named "features/a.feature" with:
      """
      Feature: some feature

      Scenario: I've declared one step and it is passing
          Given This step is passing
      """
    And a file named "features/step_definitions/cucumber_steps.js" with:
      """
      var cucumberSteps = function() {
        this.Given(/^This step is passing$/, function(callback) { callback(); });
      };
      module.exports = cucumberSteps;
      """
    And a file named "features/support/hooks.js" with:
      """
      var Stream = require('stream');

      var hooks = function () {
        this.Before(function(scenario, callback) {
          var stream = new Stream.Readable();
          stream._read = function() {};
          stream.push(new Buffer([137, 80]));
          stream.push(new Buffer([78, 71]));
          stream.push(null);

          scenario.attach(stream, 'image/png', function(error) {
            callback(error);
          });
        });
      };

      module.exports = hooks;
      """
    When I run cucumber.js with `-f json`
    Then it outputs this json:
      """
      [
        {
          "id": "some-feature",
          "name": "some feature",
          "tags": [],
          "line": 1,
          "keyword": "Feature",
          "uri": "<current-directory>/features/a.feature",
          "elements": [
            {
              "name": "I've declared one step and it is passing",
              "id": "some-feature;i've-declared-one-step-and-it-is-passing",
              "line": 3,
              "keyword": "Scenario",
              "tags": [],
              "type": "scenario",
              "steps": [
                {
                  "keyword": "Before ",
                  "hidden": true,
                  "result": {
                    "duration": "<duration>",
                    "status": "passed"
                  },
                  "arguments": [],
                  "match": {
                    "location": "<current-directory>/features/support/hooks.js:4"
                  },
                  "embeddings": [
                    {
                      "mime_type": "image/png",
                      "data": "iVBORw=="
                    }
                  ]
                },
                {
                  "name": "This step is passing",
                  "line": 4,
                  "keyword": "Given ",
                  "result": {
                    "duration": "<duration>",
                    "status": "passed"
                  },
                  "arguments": [],
                  "match": {
                    "location": "<current-directory>/features/step_definitions/cucumber_steps.js:2"
                  }
                }
              ]
            }
          ]
        }
      ]
      """

  Scenario: Attach from a before hook
    Given a file named "features/a.feature" with:
      """
      Feature: some feature

      Scenario: I've declared one step and it is passing
          Given This step is passing
      """
    And a file named "features/step_definitions/cucumber_steps.js" with:
      """
      var cucumberSteps = function() {
        this.Given(/^This step is passing$/, function(callback) { callback(); });
      };
      module.exports = cucumberSteps;
      """
    And a file named "features/support/hooks.js" with:
      """
      var hooks = function () {
        this.Before(function(scenario, callback) {
          scenario.attach("text");
          callback();
        });
      };

      module.exports = hooks;
      """
    When I run cucumber.js with `-f json`
    Then it outputs this json:
      """
      [
        {
          "id": "some-feature",
          "name": "some feature",
          "tags": [],
          "line": 1,
          "keyword": "Feature",
          "uri": "<current-directory>/features/a.feature",
          "elements": [
            {
              "name": "I've declared one step and it is passing",
              "id": "some-feature;i've-declared-one-step-and-it-is-passing",
              "line": 3,
              "keyword": "Scenario",
              "tags": [],
              "type": "scenario",
              "steps": [
                {
                  "keyword": "Before ",
                  "hidden": true,
                  "result": {
                    "duration": "<duration>",
                    "status": "passed"
                  },
                  "arguments": [],
                  "match": {
                    "location": "<current-directory>/features/support/hooks.js:2"
                  },
                  "embeddings": [
                    {
                      "mime_type": "text/plain",
                      "data": "dGV4dA=="
                    }
                  ]
                },
                {
                  "name": "This step is passing",
                  "line": 4,
                  "keyword": "Given ",
                  "result": {
                    "duration": "<duration>",
                    "status": "passed"
                  },
                  "arguments": [],
                  "match": {
                    "location": "<current-directory>/features/step_definitions/cucumber_steps.js:2"
                  }
                }
              ]
            }
          ]
        }
      ]
      """

  Scenario: Attach from an after hook
    Given a file named "features/a.feature" with:
      """
      Feature: some feature

      Scenario: I've declared one step and it is passing
          Given This step is passing
      """
    And a file named "features/step_definitions/cucumber_steps.js" with:
      """
      var cucumberSteps = function() {
        this.Given(/^This step is passing$/, function(callback) { callback(); });
      };
      module.exports = cucumberSteps;
      """
    And a file named "features/support/hooks.js" with:
      """
      var hooks = function () {
        this.After(function(scenario, callback) {
          scenario.attach("text");
          callback();
        });
      };

      module.exports = hooks;
      """
    When I run cucumber.js with `-f json`
    Then it outputs this json:
      """
      [
        {
          "id": "some-feature",
          "name": "some feature",
          "tags": [],
          "line": 1,
          "keyword": "Feature",
          "uri": "<current-directory>/features/a.feature",
          "elements": [
            {
              "name": "I've declared one step and it is passing",
              "id": "some-feature;i've-declared-one-step-and-it-is-passing",
              "line": 3,
              "keyword": "Scenario",
              "tags": [],
              "type": "scenario",
              "steps": [
                {
                  "name": "This step is passing",
                  "line": 4,
                  "keyword": "Given ",
                  "result": {
                    "duration": "<duration>",
                    "status": "passed"
                  },
                  "arguments": [],
                  "match": {
                    "location": "<current-directory>/features/step_definitions/cucumber_steps.js:2"
                  }
                },
                {
                  "keyword": "After ",
                  "hidden": true,
                  "result": {
                    "duration": "<duration>",
                    "status": "passed"
                  },
                  "arguments": [],
                  "match": {
                    "location": "<current-directory>/features/support/hooks.js:2"
                  },
                  "embeddings": [
                    {
                      "mime_type": "text/plain",
                      "data": "dGV4dA=="
                    }
                  ]
                }
              ]
            }
          ]
        }
      ]
      """
  Scenario: Attach from a step definition
    Given a file named "features/a.feature" with:
      """
      Feature: some feature

      Scenario: I've declared one step and it is passing
          Given This step is passing
      """
    And a file named "features/step_definitions/cucumber_steps.js" with:
      """
      var cucumberSteps = function() {
        this.Given(/^This step is passing$/, function(callback) {
          var world = this;
          world.scenario.attach("text");
          callback();
        });
      };
      module.exports = cucumberSteps;
      """
    And a file named "features/support/hooks.js" with:
      """
      var hooks = function () {
        this.Before(function(scenario, callback) {
          var world = this;
          world.scenario = scenario;
          callback();
        });
      };

      module.exports = hooks;
      """
    When I run cucumber.js with `-f json`
    Then it outputs this json:
      """
      [
        {
          "id": "some-feature",
          "name": "some feature",
          "tags": [],
          "line": 1,
          "keyword": "Feature",
          "uri": "<current-directory>/features/a.feature",
          "elements": [
            {
              "name": "I've declared one step and it is passing",
              "id": "some-feature;i've-declared-one-step-and-it-is-passing",
              "line": 3,
              "keyword": "Scenario",
              "tags": [],
              "type": "scenario",
              "steps": [
                {
                  "keyword": "Before ",
                  "hidden": true,
                  "result": {
                    "duration": "<duration>",
                    "status": "passed"
                  },
                  "arguments": [],
                  "match": {
                    "location": "<current-directory>/features/support/hooks.js:2"
                  }
                },
                {
                  "name": "This step is passing",
                  "line": 4,
                  "keyword": "Given ",
                  "result": {
                    "duration": "<duration>",
                    "status": "passed"
                  },
                  "arguments": [],
                  "match": {
                    "location": "<current-directory>/features/step_definitions/cucumber_steps.js:2"
                  },
                  "embeddings": [
                    {
                      "mime_type": "text/plain",
                      "data": "dGV4dA=="
                    }
                  ]
                }
              ]
            }
          ]
        }
      ]
      """
